@@ -25,7 +25,7 @@ MO file parser
 # MO file format documentation:
 # * http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-runtime/intl/gmo.h?id=v0.18.3
 # * http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-tools/src/read-mo.c?id=v0.18.3
-# * http://www.gnu.org/software/gettext/manual/html_node/MO-Files.html
+# * https://www.gnu.org/software/gettext/manual/html_node/MO-Files.html
 
 import re
 import struct
@@ -52,7 +52,7 @@ class Parser(object):
         view = memoryview(contents)
         if len(view) > 0:
             if sys.version_info >= (3, 3):
-                # http://docs.python.org/3.3/whatsnew/3.3.html#pep-3118-new-memoryview-implementation-and-buffer-protocol-documentation
+                # https://docs.python.org/3.3/whatsnew/3.3.html#pep-3118-new-memoryview-implementation-and-buffer-protocol-documentation
                 # “Accessing a memoryview element with format ‘B’ (unsigned bytes)
                 # now returns an integer […]. For returning a bytes object the view
                 # must be cast to ‘c’ first.”
@@ -77,7 +77,7 @@ class Parser(object):
         begin = at
         end = at + 4 * n
         view = self._view
-        if end > len(self._view):
+        if end > len(view):
             raise SyntaxError('truncated file')
         return struct.unpack(
             self._endian + 'I' * n,
@@ -140,6 +140,7 @@ class Parser(object):
         encoding = self._encoding
         if i == 0:
             if encoding is None and msgid == b'':
+                # FIXME? What if msgctxt is set?
                 # http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-runtime/intl/dcigettext.c?id=v0.18.3#n1106
                 match = re.search(b'charset=([^ \t\n]+)', msgstr)
                 if match is not None:
@@ -179,6 +180,9 @@ class Parser(object):
         entry.occurrences = ()
         entry.flags = ()  # https://bitbucket.org/izi/polib/issue/47
         entry.translated = lambda: True
+        entry.previous_msgctxt = None
+        entry.previous_msgid = None
+        entry.previous_msgid_plural = None
         return entry
 
 __all__ = ['Parser', 'SyntaxError']
@@ -189,8 +193,11 @@ def main():
     ap.add_argument('files', metavar='<file>', nargs='+')
     options = ap.parse_args()
     for path in options.files:
+        escaped_path = ascii(path)[1:-1]
+        print('# ' + escaped_path, end='\n\n')
         parser = Parser(path)
-        print(parser.parse())
+        for entry in parser.parse():
+            print(entry)
 
 if __name__ == '__main__':
     main()
