@@ -1,4 +1,4 @@
-# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -21,12 +21,15 @@
 import difflib
 import errno
 import inspect
+import io
+import multiprocessing as mp
 import os
 import re
 import shlex
 import signal
 import subprocess as ipc
 import sys
+import traceback
 import unittest
 
 import nose
@@ -155,7 +158,7 @@ class Plugin(nose.plugins.Plugin):
         yield TestCase(path)
 
     def wantFunction(self, func):
-        # If the plugin is being used, test_file() is reduntant.
+        # If the plugin is being used, test_file() is redundant.
         # We can't just check whether "func is test_file", because some
         # versions of nose (at least 1.1.2) reload the module.
         if func.__name__ == 'test_file' and func.__module__ == test_file.__module__:
@@ -173,40 +176,112 @@ class TestCase(unittest.TestCase):
     def __str__(self):
         return os.path.relpath(self.path)
 
-def assert_emit_tags(path, etags, *, options=()):
-    etags = list(etags)
+class SubprocessError(Exception):
+    pass
+
+def queue_get(queue, process):
+    '''
+    Remove and return an item from the queue.
+    Block until the process terminates.
+    '''
+    while True:
+        try:
+            return queue.get(timeout=1)
+            # This semi-active waiting is ugly, but here doesn't seem be any
+            # obvious better way.
+        except mp.queues.Empty:
+            if process.exitcode is None:
+                continue
+            else:
+                raise
+
+def run_i18nspector(options, path):
     commandline = os.environ.get('I18NSPECTOR_COMMANDLINE')
     if commandline is None:
+        # We cheat here a bit, becausing excec(3)ing is very expensive.
+        # Let's load the needed Python modules, and use multiprocessing to
+        # “emulate” the command execution.
+        import lib.cli
         prog = os.path.join(here, os.pardir, os.pardir, 'i18nspector')
         commandline = [sys.executable, prog]
+        queue = mp.Queue()
+        child = mp.Process(
+            target=_mp_run_i18nspector,
+            args=(prog, options, path, queue)
+        )
+        child.start()
+        [stdout, stderr] = (
+            s.splitlines()
+            for s in queue_get(queue, child)
+        )
+        child.join()
+        rc = child.exitcode
     else:
         commandline = shlex.split(commandline)
-    commandline += options
-    commandline += [path]
-    fixed_env = dict(os.environ, LC_ALL='C')
-    child = ipc.Popen(commandline, stdout=ipc.PIPE, stderr=ipc.PIPE, env=fixed_env)
-    stdout, stderr = (
-        s.decode().splitlines()
-        for s in child.communicate()
-    )
-    rc = child.poll()
-    if rc != 0:
-        if rc < 0:
-            message = ['command was interrupted by signal {sig}'.format(sig=get_signal_name(-rc))]
-        else:
-            message = ['command exited with status {rc}'.format(rc=rc)]
-        message += ['']
-        if stdout:
-            message += ['stdout:']
-            message += ['| ' + s for s in stdout] + ['']
-        else:
-            message += ['stdout: (empty)']
-        if stderr:
-            message += ['stderr:']
-            message += ['| ' + s for s in stderr]
-        else:
-            message += ['stderr: (empty)']
-        raise AssertionError('\n'.join(message))
+        commandline += options
+        commandline += [path]
+        fixed_env = dict(os.environ, PYTHONIOENCODING='UTF-8')
+        child = ipc.Popen(commandline, stdout=ipc.PIPE, stderr=ipc.PIPE, env=fixed_env)
+        stdout, stderr = (
+            s.decode('UTF-8').splitlines()
+            for s in child.communicate()
+        )
+        rc = child.poll()
+    if rc == 0:
+        return stdout
+    if rc < 0:
+        message = ['command was interrupted by signal {sig}'.format(sig=get_signal_name(-rc))]
+    else:
+        message = ['command exited with status {rc}'.format(rc=rc)]
+    message += ['']
+    if stdout:
+        message += ['stdout:']
+        message += ['| ' + s for s in stdout] + ['']
+    else:
+        message += ['stdout: (empty)']
+    if stderr:
+        message += ['stderr:']
+        message += ['| ' + s for s in stderr]
+    else:
+        message += ['stderr: (empty)']
+    raise SubprocessError('\n'.join(message))
+
+def _mp_run_i18nspector(prog, options, path, queue):
+    with open(prog, 'rt', encoding='UTF-8') as file:
+        code = file.read()
+    sys.argv = [prog] + list(options) + [path]
+    orig_stdout = sys.stdout
+    orig_stderr = sys.stderr
+    __file__ = prog
+    code = compile(code, prog, 'exec')
+    io_stdout = io.StringIO()
+    io_stderr = io.StringIO()
+    (sys.stdout, sys.stderr) = (io_stdout, io_stderr)
+    try:
+        try:
+            exec(code)
+        finally:
+            (sys.stdout, sys.stderr) = (orig_stdout, orig_stderr)
+            stdout = io_stdout.getvalue()
+            stderr = io_stderr.getvalue()
+    except SystemExit:
+        queue.put([stdout, stderr])
+        raise
+    except:
+        exctp, exc, tb = sys.exc_info()
+        stderr += ''.join(
+            traceback.format_exception(exctp, exc, tb)
+        )
+        del tb
+        queue.put([stdout, stderr])
+        sys.exit(1)
+    else:
+        queue.put([stdout, stderr])
+        sys.exit(0)
+
+def assert_emit_tags(path, etags, *, options=()):
+    etags = list(etags)
+    stdout = run_i18nspector(options, path)
     expected_failure = os.path.basename(path).startswith('xfail-')
     if stdout != etags:
         if expected_failure:
@@ -251,7 +326,7 @@ def _parse_test_header_file(file, path, *, comments_only):
 def _parse_test_headers(path):
     # <path>.tags:
     try:
-        file = open(path + '.tags', encoding='ASCII')
+        file = open(path + '.tags', encoding='UTF-8')
     except IOError as exc:
         if exc.errno != errno.ENOENT:
             raise
@@ -260,7 +335,7 @@ def _parse_test_headers(path):
             return _parse_test_header_file(file, path, comments_only=False)
     # <path>.gen:
     try:
-        file = open(path + '.gen', encoding='ASCII', errors='ignore')
+        file = open(path + '.gen', encoding='UTF-8', errors='ignore')
     except IOError as exc:
         if exc.errno != errno.ENOENT:
             raise
@@ -268,7 +343,7 @@ def _parse_test_headers(path):
         with file:
             return _parse_test_header_file(file, path, comments_only=True)
     # <path>:
-    with open(path, 'rt', encoding='ASCII', errors='ignore') as file:
+    with open(path, 'rt', encoding='UTF-8', errors='ignore') as file:
         return _parse_test_header_file(file, path, comments_only=True)
 
 def _test_file(path):
